////////////////////////////////////////////////////////////////////////////
// EditBkgd.cpp                                                           //
//
// The main source module for the application editbkgd.exe.  This
// sample demonstrates,
//   1. Handling the WM_CTLCOLOR message to change the background
//      color of a window's control.
//   2. Setting the background color of a window using the WNDCLASS
//      structure's hbrBackground.
//   3. An anomaly in TEdit which displays text half height when
//      created in a window and not a dialog in Win31 only.  See
//      the EvCreateEdit function.
//
// Written by : George Cross, Borland C++ Tech Support
// Date       : June 15, 1995
// Compiler   : Borland C++ version 5.0 but is valid for OWL 2.x up.
//
////////////////////////////////////////////////////////////////////////////

#include <owl/button.h>
#include "editbkgd.h"
#include "editbkgd.rh"
#include "myedit.hpp"
char spelltheword[] = "Spell the word:";


////////////////////////////////////////////////////////////////////////////
// Main window class TDrawWindow                                          //
////////////////////////////////////////////////////////////////////////////
DEFINE_RESPONSE_TABLE1(TDrawWindow, TWindow)
   EV_COMMAND (CM_MYHOME, EvCreateEdit),
	EV_EN_SETFOCUS	 (IDC_EDIT1, EvEnSetFocus),
	EV_EN_KILLFOCUS (IDC_EDIT1, EvEnKillFocus),
	EV_WM_CTLCOLOR,
	EV_COMMAND(200, EvButton),
END_RESPONSE_TABLE;


//------------------------------------------------------------------------//
// Constructor for TDrawWindow                                            //
//------------------------------------------------------------------------//
TDrawWindow::TDrawWindow (TWindow* parent)
				:TWindow (parent)
{
  EditBox = new TMyEdit (this,                //Create an edit control
								 IDC_EDIT1,
								 NULL,
								 100,
								 50,
								 300,
								 50,
								 0,
								 FALSE);

  //TButton*
  Button1 = new TButton (this,                //Create two buttons
								 201,
								 "Button",
								 100,
								 200,
								 50,
								 50);

            new TButton (this,
								 200,
								 "Button",
								 300,
								 200,
								 50,
								 50);
}                                             //End  TDrawWindow constructor


//------------------------------------------------------------------------//
// GetClassName assigns the TDrawWindow's registered class name.          //
// This is necessary for setting the background color in GetWindowClass.  //
//------------------------------------------------------------------------//
char * TDrawWindow::GetClassName()
{
  return "DrawWindowClass";
}


//------------------------------------------------------------------------//
// GetWindowClass is used to set this window's background color           //
//------------------------------------------------------------------------//
void TDrawWindow::GetWindowClass(WNDCLASS &wc)
{
  TWindow::GetWindowClass (wc);
  wc.hbrBackground = (HBRUSH) TBrush(TColor::LtCyan);
}


//------------------------------------------------------------------------//
// WM_CTLCOLOR response function called before all child control's are    //
// painted.   It is here that we set a control's background mode and      //
// color.
//------------------------------------------------------------------------//
HBRUSH TDrawWindow::EvCtlColor(HDC dc, HWND hWndChild, uint ctlType)
{
 if (hWndChild == EditBox->HWindow)           //If this is the edit control
 {
	 ::SetBkMode (dc, TRANSPARENT);            //Use transparent back for text
	 if (EditBox->HasFocus == FALSE)           //If the edit box has focus,
	 {
		EditBox->Attr.Style &= ~WS_BORDER;      //Remove the border
		return (HBRUSH)
				  GetStockObject (LTGRAY_BRUSH);  //Return which brush to use
	 }                                         //End if (HasFocus == FALSE)
	 else                                      //(EditBox->HasFocus == TRUE)
	 {
		EditBox->Attr.Style |= WS_BORDER;       //Put the border back on
	 }                                         //End  else
 }                                            //End  if (hWndChild ==...)
 if (hWndChild == Button1->HWindow)
    {::SetBkMode (dc, TRANSPARENT);
	  ::SetBkColor (dc, RGB(0, 255, 0));
     return (HBRUSH) GetStockObject(WHITE_BRUSH);
	 }

 return TWindow::EvCtlColor(dc,               //Call the default procedure
									 hWndChild,
									 ctlType);
}                                             //End  HBRUSH EvCtlColor(  )


//------------------------------------------------------------------------//
// EvEnSetFocus is called when the edit control gets focus                //
//------------------------------------------------------------------------//
void TDrawWindow::EvEnSetFocus ()
{
 EditBox->HasFocus = TRUE;
 EditBox->Invalidate (TRUE);
}                                             //End  EvEnSetFocus ()


//------------------------------------------------------------------------//
// EvEnKillFocus is called when the edit control gets focus               //
//------------------------------------------------------------------------//
void TDrawWindow::EvEnKillFocus ()
{
  EditBox->HasFocus = FALSE;
  EditBox->Invalidate (TRUE);
}                                             //End  EvEnKillFocus ()


//------------------------------------------------------------------------//
// EvButton - just beep
//------------------------------------------------------------------------//
void TDrawWindow::EvButton (){
	MessageBeep (-1);
}


//------------------------------------------------------------------------//
// EvCreateEdit - this function demonstrates different methods to
//                create a child edit control.
// In Win31, if you instantiate a TEdit, it will come up half height
// *unless* you set Attr.Style = WS_CHILD| WS_VISIBLE before calling
// Create().
//
// In the commented out section is the API method of creating an
// edit control and this does work correctly.
//------------------------------------------------------------------------//
void TDrawWindow::EvCreateEdit(){
   TClientDC dc(*this);
   TSize theSpellSize  =  dc.GetTextExtent(spelltheword,
                                           strlen(spelltheword));
/*
	HWND hEdit = ::CreateWindow("edit", NULL,
										 WS_CHILD|WS_VISIBLE,
                               100,120,theSpellSize.cx,theSpellSize.cy,
										 *this,NULL,*GetApplicationObject(),NULL);

	LONG editID = ::GetWindowLong(hEdit, GWL_ID);
   ::SendMessage(hEdit, WM_SETTEXT, 0, (LPARAM)(LPCTSTR)spelltheword);
*/

	TEdit*p = new TEdit   (this,                //Create an edit control
								  IDC_EDIT2,
								  NULL, 100, 120,
								  theSpellSize.cx+10,
                          theSpellSize.cy);

   // *Note*
   //  The style must be set as below in order to the edit control
   //  not to appear half height.
   //
   p->Attr.Style = WS_CHILD|WS_VISIBLE;

   p->Create();
   p->Insert(spelltheword);
}


////////////////////////////////////////////////////////////////////////////
// Application class                                                      //
////////////////////////////////////////////////////////////////////////////
class TDrawApp : public TApplication {
  public:
	 TDrawApp() : TApplication() {}
	 void InitMainWindow();
};                                            //End class TDrawApp declarat


//------------------------------------------------------------------------//
// InitMainWindow is called when the app starts up                        //
//------------------------------------------------------------------------//
void TDrawApp::InitMainWindow()
{
  SetMainWindow (                             //Assign the main window
	  new TFrameWindow (0,
							  "Edit Box Background",
							  new TDrawWindow));     //Client of TDrawWindow class
  GetMainWindow()->AssignMenu(MENU_1);
  GetMainWindow()->EnableKBHandler();
}                                             //End void InitMainWindow ()


////////////////////////////////////////////////////////////////////////////
// OwlMain. This module's main entry point.                               //
////////////////////////////////////////////////////////////////////////////
int OwlMain(int /*argc*/, char* /*argv*/ [])
{
  return TDrawApp().Run();
}                                             //End OwlMain (. .)
